//+------------------------------------------------------------------+ 
//|                                              TRADE_FUNCTIONS.mqh |
//+------------------------------------------------------------------+ 
//+------------------------------------------------------------------+ 
//| SETTING THE REQUIRED TRADE VALUES FOR CALCULATIONS               |
//+------------------------------------------------------------------+  
struct STV
  {
   int
   dgt,             // Number of decimal places
   spread,          // Difference between ask and bid prices
   stp_lvl,         // Stop level
   Leverage,        // Leverage
   TRADE_EXEMODE,   // Server trade mode
   TRADE_CALC_MODE; // Method for calculating the contract price
   //---
   double
   pnt,             // Value of one point
   ask,             // Ask price
   bid,             // Bid price
   //---
   vol_stp,         // Minimum step of changes in the volume for execution of the trade
   vol_min,         // Minimum permissible volume for position opening
   vol_max,         // Maximum permissible volume for position opening
   vol_lmt,         // Maximum permissible aggregate position volume
   //---
   fltr,            // +/- filter for freeze levels
   up_lvl,          // Calculated lower level of the stop level
   dw_lvl,          // Calculated upper level of the stop level
   //---
   tick_sz,         // Minimum price change
   tick_loss,       // Tick price for a losing position
   tick_prft;       // Tick price for a winning position
  };
//---
STV SS;
//+------------------------------------------------------------------+ 
//| FILLING THE STRUCTURE WITH TRADE VALUES                          |
//+------------------------------------------------------------------+ 
void Get_STV()
  {
   SS.dgt=(int)SymbolInfoInteger(_Symbol,SYMBOL_DIGITS);
   SS.spread=(int)SymbolInfoInteger(_Symbol,SYMBOL_SPREAD);
   SS.stp_lvl=(int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_STOPS_LEVEL);
   SS.Leverage=(int)AccountInfoInteger(ACCOUNT_LEVERAGE);
   SS.TRADE_EXEMODE=(int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_EXEMODE);
   SS.TRADE_CALC_MODE=(int)SymbolInfoInteger(_Symbol,SYMBOL_TRADE_CALC_MODE);
//---
   SS.pnt=SymbolInfoDouble(_Symbol,SYMBOL_POINT);
   SS.ask=ND_dgt(SymbolInfoDouble(_Symbol,SYMBOL_ASK),SS.dgt);
   SS.bid=ND_dgt(SymbolInfoDouble(_Symbol,SYMBOL_BID),SS.dgt);
//---
   SS.vol_stp=SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_STEP);
   SS.vol_min=SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MIN);
   SS.vol_max=SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_MAX);
   SS.vol_lmt=SymbolInfoDouble(_Symbol,SYMBOL_VOLUME_LIMIT);
//---
   SS.fltr=ND_dgt(vDgtMlt(3*SS.pnt),SS.dgt);
   SS.up_lvl=ND_dgt(SS.ask+SS.stp_lvl*SS.pnt,SS.dgt);
   SS.dw_lvl=ND_dgt(SS.bid-SS.stp_lvl*SS.pnt,SS.dgt);
//---
   SS.tick_sz=SymbolInfoDouble(_Symbol,SYMBOL_TRADE_TICK_SIZE);
   SS.tick_loss=SymbolInfoDouble(_Symbol,SYMBOL_TRADE_TICK_VALUE_LOSS);
   SS.tick_prft=SymbolInfoDouble(_Symbol,SYMBOL_TRADE_TICK_VALUE_PROFIT);
  }
//+------------------------------------------------------------------+ 
//| TRADING BLOCK                                                    |
//+------------------------------------------------------------------+ 
void TradingBlock()
  {
   int sign=-1;
   double
   lt=0.0,
   pb_tp=0.0,ps_tp=0.0,
   pb_sl=0.0,ps_sl=0.0;
//---
   ENUM_POSITION_TYPE pos_type=NULL;
//---
   Get_STV(); // Get the symbol data
//---
   sign=GetTradingSignal(0,3); // Get a signal for opening a position/increasing position volume
//---
   isPos=PositionSelect(_Symbol); // Get the flag for existence/non-existence of the position
//+------------------------------------------------------------------+ 
//| IF THERE IS A SIGNAL TO SELL                                     |
//+------------------------------------------------------------------+ 
   if(sign==0)
     {
      // Get Stop Loss and Take Profit levels
      ps_sl=GetLevelStopLoss(POSITION_TYPE_SELL,SS.bid);
      ps_tp=GetLevelTakeProfit(POSITION_TYPE_SELL,SS.bid);
      //---
      // If there is no position, open a position
      if(!isPos)
        {
         lt=NormalizeDouble(gLot,2);
         OpenPosition(lt,ORDER_TYPE_SELL,SS.bid,ps_sl,ps_tp,"");
        }
      else // If there is a position
        {
         // Get the position type
         pos_type=(ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE);
         //---
         // If there is a BUY position and reverse is enabled
         if(pos_type==POSITION_TYPE_BUY && gReverse)
           {
            lt=NormalizeDouble(gLot*2,2);
            OpenPosition(lt,ORDER_TYPE_SELL,SS.ask,ps_sl,ps_tp,"");
           }
        }
     }
//+------------------------------------------------------------------+ 
//| IF THERE IS A SIGNAL TO BUY                                      |
//+------------------------------------------------------------------+ 
   if(sign==1)
     {
      // Get Stop Loss and Take Profit levels
      pb_sl=GetLevelStopLoss(POSITION_TYPE_BUY,SS.ask);
      pb_tp=GetLevelTakeProfit(POSITION_TYPE_BUY,SS.ask);
      //---
      // If there is no position, open a position
      if(!isPos)
        {
         OpenPosition(gLot,ORDER_TYPE_BUY,SS.ask,pb_sl,pb_tp,"");
        }
      else // If there is a position
        {
         // Get the position type
         pos_type=(ENUM_POSITION_TYPE)PositionGetInteger(POSITION_TYPE);
         //---
         // If there is a SELL position and reverse is enabled
         if(pos_type==POSITION_TYPE_SELL && gReverse)
           {
            lt=NormalizeDouble(gLot*2,2);
            OpenPosition(lt,ORDER_TYPE_BUY,SS.bid,pb_sl,pb_tp,"");
           }
        }
     }
  }
//+------------------------------------------------------------------+ 
//| OPENING A POSITION                                               |
//+------------------------------------------------------------------+ 
void OpenPosition(double lots,
                  ENUM_ORDER_TYPE op_type,
                  double op_prc,
                  double sl_prc,
                  double tp_prc,
                  string nm_opr)
  {
   mytrade.SetExpertMagicNumber(0); // Set the magic number to the trade structure
   mytrade.SetDeviationInPoints(100); // Set the slippage in points
//---
// Open a position
   if(!mytrade.PositionOpen(_Symbol,op_type,lots,op_prc,sl_prc,tp_prc,nm_opr))
     {
      Print("Error when opening a position: ",Error()," - ",ErrorDesc(Error()));
     }
  }
//+------------------------------------------------------------------+ 
//| RETURNING THE TAKE PROFIT LEVEL                                  |
//+------------------------------------------------------------------+ 
double GetLevelTakeProfit(int type_pos,double level)
  {
   double tp=0.0;
//---
   if(type_pos==1) // For a SELL position (1)
     {
      // If Take Profit is required
      if(gTakeProfit>0)
        {
         // Calculate the level
         tp=ND_dgt(level-vDgtMlt((int)gTakeProfit)*SS.pnt,SS.dgt);
         //---
         // If the calculated value is lower than the limit (stop level) in the symbol specification
         if(tp<SS.dw_lvl) { return(tp); }
         else // If the value is greater or equal, correct it
           { tp=SS.dw_lvl-SS.fltr; return(tp); }
        }
     }
//---
   if(type_pos==0) // For a BUY position (0)
     {
      // If Take Profit is required
      if(gTakeProfit>0)
        {
         // Calculate the level
         tp=ND_dgt(level+vDgtMlt((int)gTakeProfit)*SS.pnt,SS.dgt);
         //---
         // If the calculated value is higher than the limit (stop level) in the symbol specification
         if(tp>SS.up_lvl) { return(tp); }
         else // If the value is lower or equal, correct it
           { tp=SS.up_lvl+SS.fltr; return(tp); }
        }
     }
//---
   return(0.0);
  }
//+------------------------------------------------------------------+ 
//| RETURNING THE STOP LOSS LEVEL                                    |
//+------------------------------------------------------------------+ 
double GetLevelStopLoss(int type_pos,double level)
  {
   double sl=0.0;
//---
   if(type_pos==1) // For a SELL position (1)
     {
      // If Stop Loss is required
      if(gStopLoss>0)
        {
         // Calculate the level
         sl=ND_dgt(level+vDgtMlt((int)gStopLoss)*SS.pnt,SS.dgt);
         //---
         // If the calculated value is higher than the limit (stop level) in the symbol specification
         if(sl>SS.up_lvl) { return(sl); }
         else // If the value is lower or equal, correct it
           { sl=SS.up_lvl+SS.fltr; return(sl); }
        }
     }
//---
   if(type_pos==0) // For a BUY position (0)
     {
      // If Stop Loss is required
      if(gStopLoss>0)
        {
         // Calculate the level
         sl=ND_dgt(level-vDgtMlt((int)gStopLoss)*SS.pnt,SS.dgt);
         //---
         // If the calculated value is lower than the limit (stop level) in the symbol specification
         if(sl<SS.dw_lvl) { return(sl); }
         else // If the value is greater or equal, correct it
           { sl=SS.dw_lvl-SS.fltr; return(sl); }
        }
     }
//---
   return(0.0);
  }
//+------------------------------------------------------------------+ 
